# -*- coding: utf-8 -*-
#
# Source: https://github.com/infra-helpers/induction-monitoring/blob/master/python/datamonitor//test_datamonitor.py
#
# Authors: Denis Arnaud, Michal Mendrygal
#

from unittest import TestCase

from elasticmock import elasticmock

from datamonitor import DataMonitor


class DataMonitoringTest(TestCase):

    @elasticmock
    def test_should_connect_and_send_object(self):
        # Variables used to test
        es_conn_str = {'host': 'localhost', 'port': 9200, 'scheme': 'http'}
        expected_document = {'unit': 'nb_of_rows', 'value': '6543'}
        es_index = 'dm-test-v0'

        # Create instance
        dm = DataMonitor()

        # Test log exceptions
        log2 = dm.set_log_level(2)
        log_try = dm.set_log_level("6")
        log_exc = dm.set_log_level("string")
        self.assertIsNone(log2)
        self.assertIsNone(log_try)
        self.assertIsNone(log_exc)
        dm.set_log_level(4)

        # Connect to Elasticsearch (ES), mocked by elasticmock here
        es_conn = dm.es_connect(es_conn_str)

        # Debug
        print(f"Connection handle for the ES cluster: {es_conn}")

        # Get the details of the ES cluster
        es_info = dm.es_info()
        self.assertIsNotNone(es_info.get('tagline'))

        # Debug
        print(f"Details for the ES cluster: {es_info}")

        # Empty index
        self.assertIsNotNone(es_index)

        # Send some simple payload to ES
        docid = dm.es_send(index=es_index, payload=expected_document)

        # Debug
        print(f"Document ID, as provided by ES: {docid}")

        # ES should return the document ID, as provided by ES
        self.assertIsNotNone(docid)

        # Extract back the same paylod from ES
        document_str = dm.es_get(index=es_index, docid=docid)
        document = None
        assert '_source' in document_str
        document = document_str['_source']

        self.assertIsNotNone(document)

        # Debug
        print(f"Document retrieved from ES: {document} (from ES answer: {document_str})")
        print(f"Document initially sent to ES: {expected_document}")

        # The document retrieved from ES should be equal to the one
        # originally sent in the step above
        self.assertEqual(expected_document, document)



# -*- coding: utf-8 -*-
#
# Source: https://github.com/infra-helpers/induction-monitoring/blob/master/python/datamonitor/datamonitor/datamonitor.py
#
# Authors: Denis Arnaud, Michal Mendrygal
#

import bz2
import inspect
import os
from itertools import repeat
from itertools import takewhile

import elasticsearch


class DataMonitor():
    """
    Helper class with a few utility methods supporting collecting KPIs
    (Key Performance Indicators) from data files and monitoring those KPI
    on tools like Elasticsearch (ES) service.
    Technically, the KPIs collected from data are meta-data.

    Log levels: 1. Critical; 2. Errors; 3: Warnings; 4: Info; 5: Verbose
    """

    def __init__(self):
        self.es_host = 'localhost'
        self.es_port = 9200
        self.es_scheme = 'http'
        self.es_user = None
        self.es_pass = None
        self.es_auth = None
        self.es_url = None
        self.es_conn = None
        self.log_level = 2

    def __str__(self):
        """
        Description of the DataMonitor instance
        """
        desc = f"DataMonitor - ES URL: {self.es_url}"
        return desc

    def set_log_level(self, level=2):
        """
        Set the log level
        """
        try:
            level_int = int(level)
            self.log_level = level_int if level_int >= 1 and level_int <= 5 else self.log_level
        except ValueError as verr:
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} ERROR - The log level parameter should be an integer between 1 and 5; it is currently {level} - {verr}")
            pass

    def get_log_pfx(self):
        """
        Derive a prefix for logging purpose
        """
        # 0 represents this line
        # 1 represents line at caller
        callerframerecord = inspect.stack()[1]

        frame = callerframerecord[0]
        info = inspect.getframeinfo(frame)
        filename = os.path.basename(info.filename)
        log_pfx = f"[DM][{filename}][{info.function}][{info.lineno}] -"
        return log_pfx

    def es_connect(self, conn=dict()):
        """
        Create and store a connection to an Elasticsearch (ES) service
        """
        if 'host' in conn:
            self.es_host = conn['host']

        if 'port' in conn:
            self.es_port = conn['port']

        if 'scheme' in conn:
            self.es_scheme = conn['scheme']

        if 'user' in conn:
            self.es_user = conn['user']

        if 'passwd' in conn:
            self.es_pass = conn['passwd']

        self.es_auth = f"{self.es_user}@" if self.es_user else ""

        self.es_url = f"{self.es_scheme}://{self.es_auth}{self.es_host}:{self.es_port}"

        if self.es_user:
            self.es_conn = elasticsearch.Elasticsearch(hosts=[self.es_host],
                                                       scheme=self.es_scheme,
                                                       port=self.es_port,
                                                       http_auth=(self.es_user,
                                                                  self.es_pass))
        else:
            self.es_conn = elasticsearch.Elasticsearch(hosts=[self.es_host],
                                                       scheme=self.es_scheme,
                                                       port=self.es_port)

        return self.es_conn

    def es_info(self):
        """
        Get the ES cluster information
        """
        es_info = self.es_conn.info()

        # Debug
        if self.log_level >= 4:
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} Details for the ES cluster ({self.es_url}): {es_info}")

        #
        return es_info

    def es_send(self, index=None, payload=dict()):
        """
        Send a JSON payload to an Elasticsearch (ES) service
        """
        doc_id = None

        if not index:
            raise Exception("An Elasticsearch (ES) index should be specified")

        doc_creation_details = self.es_conn.index(index=index, doc_type='_doc',
                                                  body=payload)
        if '_id' in doc_creation_details:
            doc_id = doc_creation_details['_id']

        # Debug
        if self.log_level >= 4:
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} Doc sent to ES ({self.es_url}/{index}) was assigned {doc_id} as doc ID")
            print(f"{log_pfx} Doc creation structure: {doc_creation_details} - Sent doc: {payload}")

        #
        return doc_id

    def es_get(self, index=None, docid=None):
        """
        Get a JSON payload from an Elasticsearch (ES) service
        """
        if not index:
            raise Exception("An Elasticsearch (ES) index should be specified")

        if not docid:
            raise Exception("An Elasticsearch (ES) document ID should be specified")

        doc_str = self.es_conn.get(index=index, id=docid)

        # Debug
        if self.log_level >= 4:
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} Got from ES ({self.es_url}/{index} for ID={docid}): {doc_str}")

        #
        return doc_str

    def calculate_nb_of_rows_in_file(self, filepath=None):
        """
        Count the number of lines in a text file.
        Inspired from https://stackoverflow.com/a/27518377/798053
        """
        o_nb_of_rows = -1

        if not filepath:
            # Report
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} Error - The filepath parameter should be set, but is not")
            return o_nb_of_rows

        does_file_exist = os.path.exists(filepath)
        if not does_file_exist:
            # Report
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} Error - The {filepath} file does not seem to exist or is not accessible")
            return o_nb_of_rows

        # Debug
        if self.log_level >= 4:
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} Counting the number of rows in the {filepath} file...")

        bufgen = None
        with bz2.open(filepath, 'rb') as f:
            bufgen = takewhile(lambda x: x,
                               (f.read(1024 * 1024) for _ in repeat(None)))
            o_nb_of_rows = sum(buf.count(b'\n') for buf in bufgen)

        # Debug
        if self.log_level >= 4:
            log_pfx = self.get_log_pfx()
            print(f"{log_pfx} The number of rows in the {filepath} file is {o_nb_of_rows}")

        return o_nb_of_rows



# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import glob
import io
import os
import re

import setuptools


def read(*names, **kwargs):
    with io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ) as fh:
        return fh.read()


setuptools.setup(
    name='datamonitor',
    version='0.0.5.post4',
    license='MIT',
    description='Python utility to monitor metadata KPI',
    long_description_content_type='text/x-rst',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.rst')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    ),
    author='Denis Arnaud',
    author_email='denis.arnaud_fedora@m4x.org',
    url='https://github.com/infra-helpers/induction-monitoring/python',
    packages=setuptools.find_packages(),
    py_modules=[os.path.splitext(os.path.basename(path))[0] for path in glob.glob('datamonitor/*.py')],
    include_package_data=True,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Utilities',
    ],
    project_urls={
        'Documentation': 'https://datamonitoring.readthedocs.io/en/latest/',
        'Changelog': 'https://datamonitoring.readthedocs.io/en/latest/changelog.html',
        'Issue Tracker': 'https://github.com/infra-helpers/induction-monitoring/issues',
    },
    keywords=[
        'data', 'monitoring', 'data-monitoring', 'quality', 'quality-assurance',
        'qa', 'kpi', 'metric', 'sensor'
    ],
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*',
    install_requires=[
        'elasticsearch',
        'elasticmock',
    ],
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    entry_points={
        'console_scripts': [
            'datamonitor = datamonitor.cli:main',
        ]
    },
)
